import os
from requests import get
from autoflpy.util.text_manipulation import contents_opener, cell_remover

try:
    from metar import Metar as mtr
except ImportError:
    print('METAR module not installed correctly')
    raise Exception('Please install the metar module before continuing.'
                    ' BASH command: $ pip install metar')

# Checks to see if os.sep is \\
if os.sep == "\\":
    # If it is then it sets the jupyter sep to be \\\\\\\\ eight slashes so \\
    jupyter_sep = "\\\\\\\\"
else:
    jupyter_sep = "/"


def metar_finder(location, year, month, day, month_end, day_end,
                 start_time_hours, end_time_hours, metar_file_path=""):
    """This grabs url if it is not already available.

    If the METAR is already cached, the cached file is returned

    If the quota limit has been reached, returns a list of variables:
    ["Quota limit reached.", location, year, month, day,
        month_end, day_end, start_time_hours, end_time_hours,
        metar_file_path]

    If no METAR data is found, returns an empty string

    If METAR data is called from the API, return data with url at the end
    """
    # End time to actual end time in hours
    print('Checking for METAR')
    end_time_hours = str(int(end_time_hours) - 1)
    # Checks to see if a metar file path is available.
    file_name = ""
    if metar_file_path != "" or metar_file_path != "data":
        # Creates a list of all the files in the METAR_file_path
        file_list = os.listdir(metar_file_path)
        # Creates a file name based off input data
        file_name = "METAR_" + location + "_" + year + month + day + \
                    "_" + year + month_end + day_end + "_" + \
                    start_time_hours + "_" + end_time_hours
        # Goes through each of the files.
        for file in file_list:
            # Checks to see if there is a cached file that matches the current
            # requested data.
            if file_name in file:
                print('Found cached METAR information')
                # Opens file.
                metar_file = open(metar_file_path + os.sep + file_name +
                                  ".txt", "r")
                # Extracts Data.
                metar_data = metar_file.read()
                # Closes file.
                metar_file.close()
                metar_data = metar_data.split("\n")
                return metar_data
    print('Getting METAR data from API')
    url = "https://www.ogimet.com/display_metars2.php?lang=en&lugar=" + \
          location + "&tipo=ALL&ord=REV&nil=SI&fmt=html&ano=" + year + \
          "&mes=" + month + "&day=" + day + "&hora=" + start_time_hours + \
          "&anof=" + year + "&mesf=" + month_end + "&dayf=" + day_end + \
          "&horaf=" + end_time_hours + "&minf=59&send=send"
    request_data = get(url)
    # Splits the data by line.
    metar_html = request_data.text.split("\n")
    # Goes through each of the lines of data.
    index = 0
    line_index = []
    # Goes through each of the lines.
    for line in metar_html:
        # If quota limit reached
        if "#Sorry, Your quota limit for " in line:
            # Return the text, Quota limit reached if the quota limit has been
            # reached.
            return (["Quota limit reached.", location, year, month, day,
                     month_end, day_end, start_time_hours, end_time_hours,
                     metar_file_path])
        # Checks to see if the lines contain that particular string.
        if "<td><font size=\"-1\"><b><pre>METAR" in line:
            # Appends data to line index.
            line_index.append(index)
        # Increments the line index.
        index += 1
    # Creates a blank list for the metar data.
    metar_data = []
    # Goes through the list and replaces HTML at the beginning and end of the
    # string.
    for index in line_index:
        # Replaces HTML at the beginning of the text.
        data = metar_html[index].replace("<td><font size=\"-1\"><b><pre>", "")
        # Replaces HTML at the end of the text.
        data = data.replace("=</pre></b></font></td>", "")
        # Appends data to a list.
        metar_data.append(data)
    # Creates an empty string to be appended to.
    full_text = ""
    # Goes through metars.
    for metar in metar_data:
        # Appends metar and new line.
        full_text += metar + "\n"
    # If there is no METAR data then the empty list is returned.
    if len(metar_data) == 0:
        return metar_data
    # TODO: find a way of shortening the URL
    # Adds url to end
    full_text += url
    # Creates a new text file to locally store and append the data to.
    metar_file = open(metar_file_path + os.sep + file_name + ".txt", "w+")
    # Extracts contents.
    metar_file.write(full_text)
    # Closes the file.
    metar_file.close()
    # Returns the list.
    metar_data.append(url)
    return metar_data


def metar_returner(metar_data, contents, months, years, number_of_flights,
                   replace_key="METAR_INFORMATION"):
    """Replaces the key word in a cell with METAR information from the day"""
    # finds the locations that the metars were recorded from.

    metar_text = ""
    for flight in range(number_of_flights):
        if "https://" not in metar_data[flight]:
            metar_text += "    \"The METARs for " + \
                          str(mtr.Metar(metar_data[flight], month=months[flight], year=years[flight]))[9:13] + \
                          " were:\\n\",\n    \"\\n\",\n"
            # Goes through the metars and creates a list of metars from that day.
        for metar in metar_data[:-1]:
            # Uses the metar function to get the data from the metar and display
            # the data labeled nicely
            metar_text += "    \"" + \
                          str(mtr.Metar(metar[6:], month=months[flight], year=years[flight]
                                        ))[14:].replace(
                              "\n", "\\n\",\n    \"\\n\",\n    \"") + \
                          "\\n\",\n     \"<br><br><br><br>\\n\",\n"
        # Adds the metar data to the text file.
        # TODO: shorten hyperlink (metar_data[-1])
        metar_text += "    \"This METAR data was from:" + metar_data[-1] + "\\n\"," + "\"\\n\",\n"

    metar_text = metar_text[:-7]  # Removes the "," + "\"\\n\",\n from the last line for json compatibility

    # Creates replacement text for the METAR key.
    metar_replacement = "\n  {\n " + \
                        "  \"cell_type\": \"markdown\",\n" + \
                        "   \"metadata\": {},\n" + \
                        "   \"source\": [\n" + \
                        "    \"" + replace_key + "\"\n" + \
                        "   ]\n" + \
                        "  },"
    # Creates some text to replace the cell.
    metar_information = "  {\n " + \
                        "  \"cell_type\": \"markdown\",\n" + \
                        "   \"metadata\": {},\n" + \
                        "   \"source\": [\n" + \
                        "    \"<h2>METAR Information</h2><a class=\\\"anchor\\\" " + \
                        "id=\\\"METAR-Information\\\"></a>\\n\",\n    \"\\n\",\n" + \
                        metar_text + \
                        "\n   ]\n" + \
                        "  },"
    contents = contents.replace(metar_replacement, metar_information)
    return contents


def no_metar_returner(location, dates, end_dates,
                      start_time_hours, end_time_hours, contents, number_of_flights,
                      replace_key="METAR_INFORMATION"):
    """Replaces the key word in a cell with note that no METAR information was
    available from the day"""
    metar_text = ""
    for flight in range(number_of_flights):
        year = str(dates[flight])[:4]
        year_end = str(end_dates[flight])[:4]
        month = str(dates[flight])[4:6]
        month_end = str(end_dates[flight])[4:6]
        day = str(dates[flight])[6:8]
        day_end = str(end_dates[flight])[6:8]
        # finds the locations that the metars were recorded from.
        metar_text += "    \"No METARs for " + str(location[flight]) + " for the date " + day + "/"+ \
                      month + "/" + year + " to the date " + day_end + "/" + \
                      month_end + "/" + year_end + \
                      " from a starting time of " + str(start_time_hours[flight]) + ":00 and an end time of " + \
                      str(int(end_time_hours[flight]) - 1) + ":59.\\n\",\n"
    metar_text = metar_text[:-2]  # Removes "," from the end for json validity.
    # Creates replacement text for the METAR key.
    metar_replacement = "\n  {\n " + \
                        "  \"cell_type\": \"markdown\",\n" + \
                        "   \"metadata\": {},\n" + \
                        "   \"source\": [\n" + \
                        "    \"" + replace_key + "\"\n" + \
                        "   ]\n" + \
                        "  },"
    # Creates some text to replace the cell.
    metar_information = "  {\n " + \
                        "  \"cell_type\": \"markdown\",\n" + \
                        "   \"metadata\": {},\n" + \
                        "   \"source\": [\n" + \
                        "    \"<h1>Metar Information <a class=\\\"anchor\\\" " + \
                        "id=\\\"Metar-Information\\\"></a></h1>\\n\",\n    \"\\n\",\n" + \
                        metar_text + \
                        "\n   ]\n" + \
                        "  },"
    contents = contents.replace(metar_replacement, metar_information)
    return contents


def metar_quota_returner(contents, file_name, location, year,
                         month, day, month_end, day_end, start_time_hours,
                         end_time_hours, metar_file_path, number_of_flights,
                         replace_key="METAR_INFORMATION"):
    """Puts metar_replacer function in to replace METAR_INFORMATION cell"""
    """Replaces the key word in a cell with METAR information from the day"""
    # TODO: Make this work for dictionary inputs
    metar_information = ""
    # Creates replacement text for the METAR key.
    metar_replacement = "\n  {\n " + \
                        "  \"cell_type\": \"markdown\",\n" + \
                        "   \"metadata\": {},\n" + \
                        "   \"source\": [\n" + \
                        "    \"" + replace_key + "\"\n" + \
                        "   ]\n" + \
                        "  },"
    # Creates some text to replace the cell.
    for flight in range(number_of_flights):
        metar_information += "  {\n " + \
                             "  \"cell_type\": \"code\",\n" + \
                             "   \"execution_count\": null,\n" + \
                             "   \"metadata\": {},\n" + \
                             "   \"outputs\": [],\n" + \
                             "   \"source\": [\n" + \
                             "    \"# METAR REPLACER{}\\n\",\n    \"\\n\",\n".format(flight) + \
                             "    \"metar_replacer(os.getcwd(), \\\"" + file_name + "\\\", \\\"" + \
                             location[flight] + "\\\", \\\"" + str(year[flight]) + "\\\", \\\"" + \
                             str(month[flight]) + "\\\", \\\"" + \
                             str(day[flight]) + "\\\", \\\"" + str(month_end[flight]) + "\\\", \\\"" +\
                             str(day_end[flight]) + \
                             "\\\", \\\"" + str(start_time_hours[flight]) + "\\\", \\\"" +\
                             str(end_time_hours[flight]) + "\\\", \\\"" + str(flight) + \
                             "\\\", \\\"" + metar_file_path.replace("\\", jupyter_sep) + \
                             "\\\")\"\n   ]\n" + \
                             "  },"
    contents = contents.replace(metar_replacement, metar_information)
    return contents


def metar_replacer(file_path, file_name, location, year, month, day,
                   month_end, day_end, start_time_hours, end_time_hours, flight,
                   metar_file_path):
    """This will replace the code with the METAR data if available."""
    # TODO: Known bug: if only one metar_replacer is run in the workbook when multiple are present, the un-run cells
    #  will be removed without any metar information being entered.
    # Finds metar data.
    metar_data = metar_finder(location, year, month, day, month_end, day_end,
                              start_time_hours, end_time_hours,
                              metar_file_path)
    if len(metar_data) == 0:
        return "No METAR data found."
    if metar_data[0] == "Quota limit reached.":
        return "Sorry, Quota limit has been reached, try again later."
    # Opens contents
    contents = contents_opener(file_path, file_name)
    # Splits contents into lines
    lines = contents.split("\n")
    # Creates an index
    index = 0
    # Creates an index showing where the last open curly bracket was.
    bracket_index = 0
    for line in lines:
        if "# METAR REPLACER{}".format(flight) in line:
            break
        if line == "  {":
            # Records position of last bracket.
            bracket_index = index
        index += 1
    lines = cell_remover(contents, "# METAR REPLACER{}".format(flight)).split("\n")
    # Lines for the METAR_returner code to replace.
    metar_information = "\n  {\n " + \
                        "  \"cell_type\": \"markdown\",\n" + \
                        "   \"metadata\": {},\n" + \
                        "   \"source\": [\n" + \
                        "    \"METAR_INFORMATION\"\n" + \
                        "   ]\n" + \
                        "  },"
    # Inserts metar_information into lines at the point where the curly bracket
    # was for the code.
    # Test
    lines.insert(bracket_index, metar_information)
    contents = ""
    # Reassembles contents from lines
    for line in lines:
        contents += line + "\n"
    contents = metar_returner(metar_data, contents, [int(month)], [int(year)], 1,
                              replace_key="METAR_INFORMATION")
    file = open(file_path + os.sep + file_name, "w+")
    # Extracts contents.
    file.write(contents)
    # Closes the file.
    file.close()
    return ("METAR data successfully included, please press save then reload" +
            " and the METAR data should appear in place of this cell.")




import pandas as pd
import pickle as pk
import os
from openpyxl import load_workbook
from autoflpy.util.metar_processing import *
from autoflpy.util.text_manipulation import *


"""
Flight Report Generation Code.
This code takes flight data in the form of an .xlsx document (generated from a
.log file in log_to_xlsx.py) or a correctly formatted .csv and plots and
formats the data to be displayed using a template .ipynb file.

@author Adrian Weishaeupl
aw6g15@soton.ac.uk 2019

Based on work done by Samuel Pearson (sp1g18@soton.ac.uk) (06-08/2019)

"""

# Checks to see if os.sep is \\
if os.sep == "\\":
    # If it is then it sets the jupyter sep to be \\\\\\\\ eight slashes so \\
    jupyter_sep = "\\\\\\\\"
else:
    jupyter_sep = "/"


def flight_log_maker(template_file_path, template_file_name,
                     flight_log_file_path, flight_data_file_path,
                     flight_data_file_names, csv_flight_data_file_path,
                     csv_flight_data_names, flight_dates, flight_numbers,
                     flight_log_file_name_header, icao_airfields, start_times_hours,
                     end_times_hours, metar_file_path, weather_data_lists, runway_data_lists,
                     include_metar):
    """This code will edit a specified template and return the result that has
    been produced after the substitution of data into the template."""
    print('Starting Flight Log Maker')
    # Sets the number of flights for iterating through the data lists
    number_of_flights = len(flight_dates)

    # loads contents.
    contents = contents_opener(template_file_path, template_file_name)

    # Formats flight_dates into a string for insertion into template
    flight_dates_str = ""
    for flight in range(number_of_flights):
        flight_date = str(flight_dates[flight])
        flight_dates_str += flight_date[6:] + "/" + flight_date[4:6] + "/" + flight_date[:4] + " and "
    flight_dates_str = flight_dates_str[:-5]

    # Inserts the dates into the contents.
    contents = contents.replace("FLIGHT_DATE", flight_dates_str)

    # Formats flight_numbers into a string for insertion into template
    flight_numbers_str = ""
    for flight in range(number_of_flights):
        flight_numbers_str += str(flight_numbers[flight]) + ", "
    flight_numbers_str = flight_numbers_str[:-2]

    # Inserts the flight number into the contents.
    contents = contents.replace("FLIGHT_NUMBER", flight_numbers_str)
    compressed_data_file_name = ""
    # Checks to see whether there is graph data or csv data that can be
    # used to plot graphs by checking if the path exists and if the path has a
    # suitable length to that it is not just greater than 1.
    directories_present = []
    for flight in range(number_of_flights):
        if (os.path.exists(flight_data_file_path + os.sep + flight_data_file_names[flight])
            is True and
            len(flight_data_file_path + os.sep + flight_data_file_names[flight]) > 1) \
            or \
            (os.path.exists(csv_flight_data_file_path + os.sep +
                            csv_flight_data_names[flight]) is True and
             len(csv_flight_data_file_path + os.sep +
                 csv_flight_data_names[flight]) > 1):
            directories_present.append(True)
        else:
            directories_present.append(False)
        # Assigns file name based on excel data
        compressed_data_file_name += str(flight_data_file_names[flight])[:-5] + "-"  # [:-5] removes the ".xslx"
    compressed_data_file_name = compressed_data_file_name[:-1]  # [:-1] removes the last "-"

    if all(directories_present):  # If all are true, carry on.
        print('Importing xlsx data')
        # This replaces the file path with the necessary information
        contents = contents.replace("PYTHON_FILE_PATH", "\\\"" +
                                    os.getcwd().replace("\\", jupyter_sep) +
                                    "\\\"")
        # This replaces the graph text label
        contents = contents.replace("GRAPH_TEXT", "")
        # This replaces the graph data label
        contents = contents.replace("\\\"# GRAPH_DATA_IMPORT\n\\\",", "")
        # This removes the label stating that a line is a graph line.
        contents = contents.replace("GRAPH_LINE", "")
        # Includes interactive Graphs.
        contents = flight_log_multiaxis_graph_contents_replacer(contents)
        # Includes interactive Graphs.
        contents = flight_log_graph_contents_replacer(contents)
    else:
        # Removes the cell importing the graph data.
        contents = cell_remover(contents, "# GRAPH_DATA_IMPORT")
        # Removes the cells containing graph data.
        contents = cell_remover(contents, "\"MULTIAXIS_GRAPH\\n\"")
        # Removes the cells containing graph data.
        contents = cell_remover(contents, "\"GRAPH\\n\"")
        # Removes the cells containing graph related text.
        contents = cell_remover(contents, "GRAPH_TEXT")
        # removes graph lines from cell
        contents = line_remover(contents, "GRAPH_LINE")

    # Assigns file name based on excel data
    compressed_data_file_path = flight_data_file_path + compressed_data_file_name + ".pkl"
    # This replaces the file path to the compressed data
    contents = contents.replace("COMPRESSED_DATA_FILE_PATH", "\\\"" +
                                (compressed_data_file_path
                                 ).replace("\\", jupyter_sep) + "\\\"")

    # Checks to see if the start and end time are in the correct format
    hours_valid = []
    for flight in range(number_of_flights):
        try:
            int(start_times_hours[flight])
            hours_valid.append(True)
        except ValueError:
            hours_valid.append(False)
        try:
            int(end_times_hours[flight])
            hours_valid.append(True)
        except ValueError:
            hours_valid.append(False)
    # Checks to see if ICAO code is valid, if not then the Metar information is
    # removed
    metar_data_list = []
    metar_generated = []
    for flight in range(number_of_flights):
        if include_metar is True:
            flight_date = str(flight_dates[flight])
            if (icao_airfields[flight] != "data" or icao_airfields[flight] != "") and all(hours_valid) is True:
                # Retrieves METAR data.
                metar_data_list.append(metar_finder(icao_airfields[flight], str(flight_date[:4]),
                                                    str(flight_date[4:6]), str(flight_date[6:8]),
                                                    str(flight_date[4:6]), str(flight_date[6:8]),
                                                    str(start_times_hours[flight]), str(end_times_hours[flight]),
                                                    metar_file_path))
                metar_generated.append(True)
            else:
                metar_generated.append(False)

    # Generates a list of months and years from flight_dates to be used in the metar functions
    days = []
    months = []
    years = []
    for flight in range(number_of_flights):
        days.append(int(str(flight_dates[flight])[6:8]))
        months.append(int(str(flight_dates[flight])[4:6]))
        years.append(int(str(flight_dates[flight])[:4]))

    if all(metar_generated) is not False and include_metar is True:
        for metar_data in metar_data_list:
            # Checks to see if METAR data is available.
            if len(metar_data) != 0:
                # Checks to see if the quota limit has been reached.
                if metar_data[0] == "Quota limit reached.":
                    # If the limit has been reached then it puts in a line of code
                    # to re-try when the quota limit has been reached.
                    contents = metar_quota_returner(contents,
                                                    flight_log_file_name_header +
                                                    compressed_data_file_name +
                                                    ".ipynb", icao_airfields,
                                                    years,
                                                    months,
                                                    days,
                                                    months,
                                                    days,
                                                    start_times_hours,
                                                    end_times_hours,
                                                    metar_file_path, number_of_flights,
                                                    replace_key="METAR_" +
                                                                "INFORMATION")
                    # Replaces JFLTS METAR text with nothing if data is available.
                    contents = contents.replace("METAR_LINE", "")
                    contents = contents.replace("METAR_TEXT", "")
                else:
                    # Includes METAR data into the contents.
                    contents = metar_returner(metar_data, contents,
                                              months,
                                              years, number_of_flights,
                                              replace_key="METAR_INFORMATION")
                    # Replaces JFLTS METAR text with nothing if data is available.
                    contents = contents.replace("METAR_LINE", "")
                    contents = contents.replace("METAR_TEXT", "")
            else:
                contents = no_metar_returner(icao_airfields, flight_dates,
                                             flight_dates,
                                             start_times_hours, end_times_hours,
                                             contents, number_of_flights,
                                             replace_key="METAR_INFORMATION")
                # Replaces JFLTS METAR text with nothing if data is available
                contents = contents.replace("METAR_LINE", "")
                contents = contents.replace("METAR_TEXT", "")
    else:
        # Removes METAR related cells and lines.
        contents = cell_remover(contents, "METAR_INFORMATION")
        contents = line_remover(contents, "METAR_LINE")
        contents = cell_remover(contents, "METAR_TEXT")

    # Formats dictionary appropriately to be put into the jupyter notebook as text
    weather_information = ""
    runway_information = ""
    for flight in range(number_of_flights):
        weather_information += dictionary_reader(weather_data_lists[flight], debug_name="Weather data",
                                                 units_present=True, flight_index=flight,
                                                 flight_number=(str(flight_numbers[flight] + " " +
                                                                str(flight_dates[flight])))) + ","
        runway_information += dictionary_reader(runway_data_lists[flight], debug_name="Runway data",
                                                units_present=False, flight_index=flight,
                                                flight_number=(str(flight_numbers[flight] + " " +
                                                               str(flight_dates[flight])))) + ","
    weather_information = weather_information[:-1]  # Removes the last "," for json formatting
    runway_information = runway_information[:-1]  # Removes the last "," for json formatting

    if "empty_dictionary" not in weather_information:  # If information is present, add it to the content.
        contents = contents.replace("\"WEATHER_INFORMATION\"", "\"<h2>Weather Information</h2><a class=\\\"anchor\\\"" +
                                    " id=\\\"Weather-Information\\\"></a>\\n\"" + ",\n   \"\\n\",\n   " +
                                    weather_information)  # Creates a formatted title
        contents = contents.replace("WEATHER_LINE", "")
        contents = contents.replace("WEATHER_TEXT", "")
    else:
        # Removes Weather related cells and lines.
        contents = cell_remover(contents, "WEATHER_INFORMATION")
        contents = line_remover(contents, "WEATHER_LINE")
        contents = cell_remover(contents, "WEATHER_TEXT")
        contents = contents.replace("WEATHER_INFORMATION", "")

    if "empty_dictionary" not in runway_information:  # If information is present, add it to the content.
        contents = contents.replace("\"RUNWAY_INFORMATION\"", "\"<h2>Runway Information</h2><a class=\\\"anchor\\\"" +
                                    " id=\\\"Runway-Information\\\"></a>\\n\"" + ",\n   \"\\n\",\n   " +
                                    runway_information)  # Creates a formatted title
        contents = contents.replace("RUNWAY_LINE", "")
        contents = contents.replace("RUNWAY_TEXT", "")
    else:
        # Removes runway related cells and lines.
        contents = cell_remover(contents, "RUNWAY_INFORMATION")
        contents = line_remover(contents, "RUNWAY_LINE")
        contents = cell_remover(contents, "RUNWAY_TEXT")
        contents = contents.replace("RUNWAY_INFORMATION", "")

    # Creates a new flight log from the contents
    flight_log_creator(contents, flight_log_file_path, compressed_data_file_name, flight_log_file_name_header)
    print('Pickling data')
    # Compresses the flight data for faster loading
    compile_and_compress(flight_data_file_path, flight_data_file_names,
                         csv_flight_data_file_path,
                         csv_flight_data_names, compressed_data_file_path)
    print('Flight log maker finished')


def flight_data(file_path, file_name):
    """This imports the data excel using pandas"""
    # Excel file.
    file_path_with_name = file_path + file_name
    # # Extracts data from each sheet.
    frame = pd.read_excel(file_path_with_name, sheet_name=None, engine='openpyxl')
    frame_list = []
    for data in frame.keys():
        frame_list.append(frame[data])
    return frame_list


def flight_data_and_axis(new_frames):
    """Returns list of lists with the following structure:
    [[flight, [data source, [name, unit, data],[name, unit, data]]], [flight, [data source, [name, unit, data],
    [name, unit, data]]]]."""
    # TODO: try rewriting this code in a more legible way.
    # Creates an empty list for all the data.
    values_list = []
    # Finds the flight date and number from the data
    flight_identifier = new_frames[0].columns[0][-16:]
    # Checks through all the data frames
    for frame in new_frames:
        # Goes through all the columns
        data_lists = []
        data_lists.append(frame.columns[0].split("_")[-3])
        for column in frame.columns:
            # Converts values from data frame into a list.
            y = frame[column].tolist()
            # This code creates the y-axis label.
            # Creates a name
            name = ""
            unit = ""
            # b is used to place a capital letter for the first letter.
            b = 0
            # This checks the length of the columns to check if units need to
            # be included.
            if len(column.split("_")) == 5:
                # a is used to check if the underscore has been reached.
                a = 0
                # unit_present is used to indicate that there is a unit.
                unit_present = True
            else:
                if len(column.split("_")) == 4:
                    # This indicates that there are no units for that
                    # particular value.
                    unit_present = False
                    a = 0
                else:
                    a = 5 - len(column.split("_"))

            # This for loop creates the name and unit data.
            for letter in column:
                if b == 0:
                    # Adds an uppercase letter to the start of the name
                    name = letter.upper()
                    # Sets b to 1 to indicate that a capital letter has been
                    # inserted.
                    b = 1
                else:
                    if a <= 0:
                        # Replaces underscore with a space.
                        if letter != "_":
                            # Adds letters to build up name.
                            name = name + letter
                        else:
                            # Increment a
                            a += 1
                            # If the name has more than one word in it, use
                            # a space as a separator
                            if a <= 0:
                                name += " "
                    else:
                        # TODO: unit_present is only set when a is 0 but only accessed when a is > 0. This is a problem.
                        if letter != "_" and unit_present is True:
                            unit = unit + letter
                        elif unit_present is False:
                            unit = "no unit"
                            break
                        else:
                            break
            # Replaces per with the superscript notation.
            if "per" in unit:
                # splits unit into list about per.
                unit_split = unit.split("per")
                # Creates a blank list for counting the number of occurrences.
                # of a particular denominator.
                denominators = []
                # a is used to check if the value is a numerator or a
                # denominator.
                a = 0
                for count in range(1, len(unit_split)):
                    # Checks through denominator list
                    for item in denominators:
                        # Checks if there is already a denominator with that
                        # unit.
                        if unit_split[count] in item[0]:
                            # If there is then it increases the power of that
                            # unit.
                            item[1] = item[1] + 1
                            # Sets the check to 1 to show that this unit has
                            # appeared before.
                            a = 1
                    # Checks if the denominator has appeared before.
                    if a == 1:
                        a = 0
                    else:
                        # If not it creates a new entry.
                        denominators.append([unit_split[count], 1])
                # recreates unit.
                unit = unit_split[0]
                # Goes through each denominators and formats adds it to the
                # unit list.
                for denominator in denominators:
                    # Puts units in format with powers
                    unit = unit + denominator[0] + "$^{-" + \
                           str(denominator[1]) + "}$"
            data_lists.append([name, unit, y])
        values_list.append(data_lists)
    return [flight_identifier, values_list]


def flight_log_creator(contents, file_path, file_name_data,
                       file_name_header="Flight_Log_"):
    """This creates or overwrites a file with the name file_name_header date
     flight_numbers.ipynb  and fills the file with the
    contents provided as an input."""
    # Creates file with name
    print('Creating new flight log')
    file = open(file_path + os.sep + file_name_header + file_name_data + ".ipynb", "w+")
    # Extracts contents.
    file.write(contents)
    # Closes the file.
    file.close()


def flight_data_time_sorter(frame_list):
    """Splits the data by the title containing time."""
    a = 0
    # Creates a blank dictionary for the time columns.
    renamed_time_columns = {}
    # Empty list to be populated by columns that contain time.
    time_columns = []
    # Empty list for new frames to fill.
    new_frames = []
    # Edits one frame at a time.
    for frame_data in frame_list:
        # Empty list to be populated by columns that contain time more than
        # once.
        frame_columns_list = []
        # Columns is a list of columns.
        columns = frame_list[a].columns
        # Increments a
        a = a + 1
        # Loops through each column to see if it contains the word time.
        for column_data in columns:
            # Checks to see if the word time appears in the column.
            try:
                if "action_time" in column_data.lower():
                    # Exceptons are passed here.
                    pass
                elif "time" in column_data.lower():
                    # If time appears then it is appended to the list time_columns.
                    time_columns.append(column_data)
                    # Divides column by 1*10^6 to return it to seconds.
                    frame_data[column_data] = frame_data[column_data].div(10 ** 6)
                    # Adds columns in this data frame that contain the word time to
                    # a list.
                    frame_columns_list.append(column_data)
                    # appends replacement column to dictionary.
                    renamed_time_columns[column_data] = (column_data.replace("_US_", "_s_"))
            except TypeError:  # This exception is for the user collected environmental data time
                if "dummy_time" in column_data.lower():
                    frame_columns_list.append(column_data)
                else:
                    pass

        # Creates a copy of the columns to delete parts from.
        frame_copy = frame_data.copy()
        # The variable b will be used to divide the list.
        b = 0
        # This variable is used to trigger the appending of the data frames to
        # the list data frames.
        d = 0
        # Checks through each of the columns.
        frame_duplicate = None
        for column_data in columns:
            if b == 0:
                frame_duplicate = frame_copy.copy()
                b = 1
                # Checks to see if column is a time column.
                for time_column in time_columns:
                    if column_data == time_column:
                        d = 1
                # Removes frames
                frame_copy = frame_copy.drop(labels=column_data, axis=1)
                if d == 1:
                    # Removes columns from the frame_duplicate list
                    for column_drop in frame_copy.columns:
                        frame_duplicate = frame_duplicate.drop(labels=column_drop, axis=1)
                    # Checks if time appears in column being checked.
                    new_frames.append(frame_duplicate)
                    b = d = 0
            else:
                # Checks to see if column is a time column.
                for time_column in time_columns:
                    if column_data == time_column:
                        d = 1
                # Removes frames
                frame_copy = frame_copy.drop(labels=column_data, axis=1)
                if d == 1:
                    # Removes columns from the frame_duplicate list
                    for column_drop in frame_copy.columns:
                        frame_duplicate = frame_duplicate.drop(labels=column_drop, axis=1)
                    # Checks if time appears in column being checked.
                    new_frames.append(frame_duplicate)
                    b = d = 0
    # Creates a blank list for the frames with the seconds unit fixed.
    arranged_frames = []
    # Renames columns and changes the units of time from US (microseconds) to s
    for frame_data in new_frames:
        # Replaces the unit
        frame_data.rename(columns=renamed_time_columns, inplace=True)
        # Appends the frame.
        arranged_frames.append(frame_data)
    return arranged_frames


def file_type_finder(file_path, extension):
    """ Returns files of a particular type given an input file path
    and extension."""
    # Finds list of files in file_path.
    files = os.listdir(file_path)
    # Reversed extension
    reversed_extension = extension[::-1]
    # removes the . from the file extension.
    reversed_extension = reversed_extension[:-1]
    # Creates empty list of files to check.
    required_files = []
    # Looks through each file individually.
    for file in files:
        # Empties rev_file_extension.
        rev_file_extension = ""
        # a is used to indicate that the . before the file extension has been
        # reached, it is set to 0 to reset it.
        a = 0
        for character in reversed(file):
            # Adds characters from file name to rev_file_extension if the dot
            # has not been reached yet.
            if character != "." and a == 0:
                # Adds letters to filenames.
                rev_file_extension = rev_file_extension + character
            else:
                a = 1
        # If the . has been reached and the file extension matches the
        # requested extension then the file name is added to a list of suitable
        # files.
        if a == 1 and rev_file_extension == reversed_extension:
            required_files.append(file)
    return required_files


def date_and_flight_number(frames):
    """This function finds the date and flight number from the headings in
    the list of frames provided by the function flight data"""
    # Starts at the data frame (first sheet).
    for frame_date in frames:
        # Scans through the columns.
        for column_date in frame_date.columns:
            # Splits column name by the underscore.
            lines = column_date.split("_")
            # Finds length of columns.
            a = len(lines)
            # If columns are too short to have name, unit, date and Flight
            # Number then it tries the next column.
            if a >= 3:
                # Checks the penultimate value.
                date = lines[a - 2]
                # Removes the word flight from the final value to leave just
                # the Flight Number
                flight_number = lines[a - 1][6:]
                # Returns the Date and the Flight Number
                return date, flight_number


def csv_frame(flight_data_file_path, csv_flight_data_name):
    """Takes file path of csv flight data and returns a pandas data frame_micro."""
    # Creates file path from graph file path and file name
    file_path = flight_data_file_path + os.sep + csv_flight_data_name
    if csv_flight_data_name == "" or flight_data_file_path == "":
        return pd.DataFrame()
    # Reads CSV
    frame_micro = pd.read_csv(file_path)
    return frame_micro


def compile_and_compress(flight_data_file_path, flight_data_file_name,
                         csv_data_file_path, csv_data_file_name,
                         comp_data_file_path):
    """
    This is used to compile all the entered data. This is then pickled and saved for faster loading.

    flight_data_file_names and csv_data_file_name are of type list.
    """
    values_list = []
    for data_set in range(len(flight_data_file_name)):
        # Excel Sheets
        print(flight_data_file_name[data_set])
        frame_list = flight_data(flight_data_file_path, flight_data_file_name[data_set])
        # Retrieves csv flight data if present
        if csv_data_file_name != ['']:
            csv_flight_data_frame = csv_frame(csv_data_file_path,
                                              csv_data_file_name[data_set])
            # Appends csv frame to flight data from pixhawk
            frame_list.append(csv_flight_data_frame)
        # Sorts frames by time
        sorted_frames = flight_data_time_sorter(frame_list)
        values = flight_data_and_axis(sorted_frames)
        # Creates a list of all the values.
        values_list.append(values)
    # Compresses (pickles) the data and saves it in the excel files folder.
    pk.dump(values_list, open(comp_data_file_path, "wb"))
    print('Pickling finished')


def dictionary_reader(dictionary, debug_name="Dictionary data", units_present=False, flight_index=0,
                      flight_number=""):
    """Takes a dictionary of data and outputs a formatted string in the form of a table"""
    # Splits the dictionary into lists of values and keys
    dictionary_keys = list(dictionary.keys())
    dictionary_values = list(dictionary.values())
    if dictionary == {'': 'N/A'}:
        text = "\"empty_dictionary \\n\""
    else:

        try:
            text = ""

            units = []
            names = []
            if units_present is True:
                # Removes the _ from the names and separates the units from the variable names
                for key in dictionary_keys:
                    units.append(str(key).split("_")[-1])
                    names.append(str(key).split("_")[:-1])
            else:
                # Removes the _ from the names
                for key in dictionary_keys:
                    names.append(str(key).split("_"))

            # Joins up variable names in case they are two words
            joined_names = []
            for name_array in names:
                name = ""
                for part in name_array:
                    name += part + " "
                joined_names.append(name)

            if units_present is True:
                joined_names, units, dictionary_values = zip(*sorted(zip(
                    joined_names, units, dictionary_values)))  # Sort alphabetically
            else:
                joined_names, dictionary_values = zip(*sorted(zip(
                    joined_names, dictionary_values)))  # Sort alphabetically

            if flight_index == 0:
                # Create a header for the table
                header = "\"\\n\", \" | Flight |"
                if units_present is True:
                    for data_item in range(len(dictionary_keys)):
                        header += " " + str(joined_names[data_item]) + " (" + str(units[data_item]) + ") |"
                else:
                    for data_item in range(len(dictionary_keys)):
                        header += " " + str(joined_names[data_item]) + " |"
                text = header + "\", \n "
                # Creates the next empty row
                counter = 0
                text += "\"\\n\", \" | --- |"
                while counter <= len(dictionary_keys) - 1:
                    text += " --- |"
                    counter += 1
                text += "\", \n   "
            else:
                text = ""

            # Adds data to the table
            text += "\"\\n |" + " Flight " + str(flight_number) + "|"
            for data_item in range(len(dictionary_keys)):
                text += " " + str(dictionary_values[data_item]) + " |"

            text += "\"  \n   "

        except ValueError:
            print("{0} not entered or in an incorrect format in the Input_File.json. ".format(debug_name) +
                  "Format should be \"variable_unit\": \"value\"")
            text = ""

    return text


def multi_dictionary_data_formatter(dictionaries, flight_dates, debug_name):
    """Takes in a dictionaries of data with possibly multiple data sets separated by a "," and
    formats them into lists of dictionaries which each contain one data set.

    flight_dates is used to determine the number of flights

    e.g.: dictionaries{key} = {value_dataset_1, value__dataset_2}
    into:
    [{key}=value_dataset_1, {key}=value_dataset_2]

    returns a list of dictionaries
    """

    dictionary = []
    if len(dictionaries.keys()) == 0:
        # If the lists are empty, create blank templates
        for time in range(len(flight_dates)):
            dictionary.append({"": "N/A"})
    else:
        # Removes any blank fields in the dictionaries:
        for key in dictionaries.keys():
            if dictionaries[key] == "":
                dictionaries = remove_dictionary_key(dictionaries, key)
            else:
                pass

        # Checks that the dictionary data lengths are equal to the number of flights
        for item in dictionaries.values():
            if len(item.replace(" ", "").split(",")) == len(flight_dates):
                pass
            else:
                raise IndexError("{} data content does not match the number"
                                 " of flights entered ({}).".format(debug_name, len(flight_dates)))

        # Formats dictionaries data as lists of data dictionaries
        for flight in range(len(flight_dates)):
            # Creates a new dictionary for each set of data
            dictionary_data_temp = {}
            for key in dictionaries.keys():
                dictionary_data_temp[key] = dictionaries[key].replace(" ", "").split(",")[flight]
            dictionary.append(dictionary_data_temp)

    return dictionary


def remove_dictionary_key(dictionary, key):
    """Used to delete a key in a dictionary"""
    new_dictionary = dict(dictionary)
    del new_dictionary[key]
    return new_dictionary


def multi_string_data_formatter(data_string, flight_dates, debug_name):
    """Takes a string of multiple inputs separated by a "," and separates this into a list of multiple inputs.

    flight_dates a list used to verify that enough data has been entered (is equal to len(flight_data)).

    e.g. data_string = "1234,4567"
    output = [1234, 4567]
    """

    data_list = str(data_string).replace(" ", "").split(",")
    if len(data_list) != len(flight_dates) and data_list != [""]:
        raise IndexError("{} data entered does not have the same length as the number of flights ({})"
                         .format(debug_name, len(flight_dates)))

    return data_list



# -*- coding: utf-8 -*-
"""
Finds the nearest ICAO from the generated data.

@author: Adrian Weishaeupl
aw6g15@soton.ac.uk 2019

runways.csv downloaded from https://github.com/sobester/ADRpy/blob/master/ADRpy
/data/runways.csv
"""

import numpy as np
import os
import pandas as pd
from openpyxl import load_workbook


def multi_icao_finder(file_path, file_names, flight_names):
    """Runs the icao_finder for multiple file_names in the form of lists from the same file_path

    returns a list of ICAO codes found for the respective flights
    """
    icaos = []
    for flight in range(len(flight_names)):
        print("Finding ICAO for {}".format(flight_names[flight]))
        icaos.append(icao_finder(file_path, file_names[flight]))
    return icaos


def icao_finder(file_path, file_name):
    """Puts together sub-functions to find the UAV's nearest ICAO"""
    # Finds airport latitudes and longitudes
    airport_data = airport_lat_long()
    # Finds uav latitude and longitude at switch on
    uav_location = uav_lat_long(file_path, file_name)
    # Finds the ICAO code of the closest airfield
    nearest_icao = closest_icao(uav_location, np.array([airport_data[:, 1],
                                                       airport_data[:, 2]]))
    icao = airport_data[nearest_icao, 0]
    print('Nearest ICAO = ' + str(icao))
    return icao


def closest_icao(uav_lat_long_clst, airport_lat_long_clst):
    # Finds the closest airport to the latitudes and longitude of the UAV's
    # start location.
    # NOTE: selected airport may not have reliable METAR information.
    # based on code from: https://codereview.stackexchange.com/questions/28207/
    # finding-the-closest-point-to-a-list-of-points
    # Turns the airport list into a numpy array.
    airport_lat_long_clst = np.asarray(airport_lat_long_clst)
    # Finds the square distance to the closest airport - this is in terms of
    # difference in latitude and longitude (always positive).
    print('Calculating closest distance')
    dist_2 = np.sum((np.subtract(airport_lat_long_clst, uav_lat_long_clst)) ** 2, axis=0)
    if np.min(dist_2) > 0.05:
        print('Nearest airport weather data is not reliable')
    # Returns the array index of the closest airport
    return np.argmin(dist_2)


def airport_lat_long():
    """Returns a list of airport Latitudes and longitudes from runways.csv"""
    # Creates the file path and names the csv.
    print('Importing runway data. Adapted from: '
          'https://github.com/sobester/ADRpy')
    # Runways data is taken from ADRpy and has empty location cells filled with
    # very large numbers to eliminate the likelihood of them being picked.
    file_path = os.path.join(os.path.dirname(__file__)[:-5], "data",
                             "runways.csv")
    # Creates pandas data frames for the data required.
    data = pd.read_csv(file_path)
    information = np.array([data['airport_ident'], data['le_latitude_deg'],
                            data['le_longitude_deg']])
    return np.transpose(information)


def uav_lat_long(file_path, file_name):
    """Returns the UAV's latitude and longitude from the start of the flight
    (from the log files)"""
    # Reads from the xls document
    print('Finding UAV position')
    uav_log_file = load_workbook(file_path + file_name, read_only=True)
    # Finds the GPS spreadsheet
    sheet = uav_log_file['GPS']
    # Finds the first entry for the latitude and longitude
    # NOTE: this might be unreliable for different input formats.
    uav_lat = float(sheet['F2'].value)
    uav_long = float(sheet['G2'].value)
    uav_position = np.array([[uav_lat], [uav_long]])
    print('UAV position = ' + str(uav_lat) + ', ' + str(uav_long))
    return uav_position



# -*- coding: utf-8 -*-
"""
Unit tests for the text manipulation function
Created on Tue Feb 18 14:53:20 2020

@author Adrian Weishaeupl (aw6g15@soton.ac.uk)
"""

import unittest
import os
import json
from autoflpy.util import text_manipulation
from autoflpy.util.testing.check_str_in_content import *


class TestTextManipulation(unittest.TestCase):

    def setUp(self):
        # Sets up all of the test variables and locations to be used
        # throughout.
        self.base_path = os.path.join(os.path.dirname(__file__),
                                      "test_files") + os.sep
        self.base_path = self.base_path.replace(os.sep, "/")
        # Imports data from template.
        with open(self.base_path + 'test_Input_File.json') as file:
            self.data = json.load(file)
        self.template_file_path = self.base_path
        self.template_file_name = self.data["flight_log_generator_input"][
                "template_file_name"]

    def test_contents_opener(self):
        contents = text_manipulation.contents_opener(self.template_file_path,
                                                     self.template_file_name)
        test_string = (contents[370:430])
        test_string_true = 'o view images, place the images in a folder' +\
            ' called images (a'
        self.assertEqual(test_string_true, test_string)

    def test_flight_log_graph_contents_replacer(self):
        # Function being tested is expected to replace all fields with
        # single axis graphs.
        # Generates content.
        contents = text_manipulation.contents_opener(self.template_file_path,
                                                     self.template_file_name)
        # Runs the flight_log_content_replacer
        contents = text_manipulation.flight_log_graph_contents_replacer(contents)
        # Checks for values replaced by the function
        check_x_lim = check_str_in_content('x_limits', contents)
        check_y_lim = check_str_in_content('y_limits', contents)
        check_graph_plotter = check_str_in_content('graph_plotter', contents)
        self.assertEqual(26, check_x_lim)
        self.assertEqual(26, check_y_lim)
        self.assertEqual(13, check_graph_plotter)

    def test_flight_log_multiaxis_graph_contents_replacer(self):
        # Function being tested is expected to replace all fields with
        # multi-axis graphs.
        # Generates content.
        contents = text_manipulation.contents_opener(self.template_file_path,
                                                     self.template_file_name)
        # Runs the flight_log_content_replacer
        contents = text_manipulation. \
            flight_log_multiaxis_graph_contents_replacer(contents)
        # Checks for values replaced by the function
        check_x_lim = check_str_in_content('x_limits', contents)
        check_y_lim = check_str_in_content('y_limits', contents)
        check_graph_plotter = check_str_in_content('graph_plotter', contents)
        self.assertEqual(6, check_x_lim)
        self.assertEqual(12, check_y_lim)  # Note: multiple y axis inputs.
        self.assertEqual(3, check_graph_plotter)

    def test_cell_remover(self):
        # Defines keys to be tested in the cell remover
        keys = ['METAR_INFORMATION', 'GRAPH',
                'MULTIAXIS_GRAPH']
        for key in keys:
            # Generates content
            content = text_manipulation.contents_opener(self.template_file_path,
                                                        self.template_file_name)
            # Counts the keys in the content
            content_count = check_str_in_content(key, content)
            # Checks that the keys are present
            self.assertNotEqual(0, content_count)
            # Runs cell remover
            content = text_manipulation.cell_remover(content, key)
            # Counts Key occurrences
            content_count = check_str_in_content(key, content)
            # Checks that the cell has been removed
            self.assertEqual(0, content_count)

    def test_line_remover(self):
        # Defines keys to be tested in the line remover
        keys = ['METAR_INFORMATION', 'GRAPH',
                'MULTIAXIS_GRAPH', 'GRAPH_LINE']
        for key in keys:
            # Generates content
            content = text_manipulation.contents_opener(self.template_file_path,
                                                        self.template_file_name)
            # Counts how many times the key is present
            content_count = check_str_in_content(key, content)
            # Checks that the key is present
            self.assertNotEqual(0, content_count)
            # Runs line remover
            content = text_manipulation.line_remover(content, key)
            # Counts Key occurrences
            content_count = check_str_in_content(key, content)
            # Checks that the line has been removed
            self.assertEqual(0, content_count)

    def test_contents_opener(self):
        """Tests the contents_opener()"""
        # Assigns variables
        file_path = self.base_path
        file_name1 = "test_data_sources.txt"
        file_name2 = "METAR_EGHE_20190123_20190123_9_9.txt"

        # Assigns outputs
        expected_output1 = "Data Sources specified here will be placed in the excel document\nGPS\nAOA\nCTUN\n" \
                           "RCIN\nARSP\nBARO\nATT\nVIBE"
        expected_output2 = "METAR EGHE 230950Z 34017KT 9999 SCT025 08/03 Q1007\nMETAR EGHE 230920Z 34017KT 9999" \
                           " SCT022 08/02 Q1006\nhttps://www.ogimet.com/display_metars2.php?lang=en&lugar=EGHE&t" \
                           "ipo=ALL&ord=REV&nil=SI&fmt=html&ano=2019&mes=01&day=23&hora=9&anof=2019&mesf=01&dayf=" \
                           "23&horaf=9&minf=59&send=send"

        # Runs method
        output1 = text_manipulation.contents_opener(file_path, file_name1)
        output2 = text_manipulation.contents_opener(file_path, file_name2)

        # Checks results match
        self.assertEqual(output1, expected_output1)
        self.assertEqual(output2, expected_output2)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Unit tests for the flight_log_code.py.

@author Adrian Weishaeupl (aw6g15@soton.ac.uk)
"""

import unittest
import os
import json
import sys
from autoflpy.util import flight_log_code
from datetime import datetime
from autoflpy.util import nearest_ICAO_finder
from autoflpy.util.testing.check_str_in_content import *


def notebook_sample_code(flight_data_file_path, flight_data_file_name,
                         csv_data_file_path, csv_flight_data_name):
    """Sample flight log code for testing purposes"""
    # GRAPH_DATA_IMPORT

    # Creates a base_path to the test files.
    base_path = os.path.dirname(__file__) + os.sep

    # Creates a link to where the code is stored.
    sys.path.append(base_path[:-11])

    # file_path of the flight data.
    # flight_data_file_path = base_path
    # Excel File name
    # flight_data_file_name = "test_xlsx.xlsx"
    # CSV File name
    # csv_flight_data_name = "test_csv.CSV"
    # csv Data file path
    # csv_data_file_path = base_path
    # Excel Sheets
    frame_list = flight_log_code.flight_data(flight_data_file_path,
                                             flight_data_file_name)
    # A list containing the date first and then the flight number
    date_and_flight_number = flight_log_code.date_and_flight_number(frame_list)
    # Retrieves csv flight data
    csv_flight_data_frame = \
        flight_log_code.csv_frame(csv_data_file_path,
                                  csv_flight_data_name)
    # Appends csv frame to flight data from pixhawk
    frame_list.append(csv_flight_data_frame)
    # Sorts frames by time
    sorted_frames = flight_log_code.flight_data_time_sorter(frame_list)
    # Creates a list of all the values.
    values_list = flight_log_code.flight_data_and_axis(sorted_frames)
    return (frame_list, date_and_flight_number, csv_flight_data_frame,
            sorted_frames, values_list)


class TestFlightLogCode(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # Runs once before any tests
        # Defines the base path
        base_path = os.path.join(os.path.dirname(__file__), "test_files"
                                 ) + os.sep
        # Tidies base path
        base_path = base_path.replace(os.sep, "/")
        # Opens the input file and reads content
        with open(base_path + 'test_Input_File.json') as file:
            data = json.load(file)
        # Defines variables
        flight_data_file_path = base_path
        flight_data_file_name = "test_xlsx.xlsx"
        csv_flight_data_file_path = base_path + data[
            "flight_log_generator_input"]["csv_flight_data_file_path"]
        csv_flight_data_name = data["flight_log_generator_input"
        ]["csv_flight_data_name"]
        # Creates a global variable to be used in the testing
        global notebook_results
        # Populates the global variable
        notebook_results = \
            notebook_sample_code(flight_data_file_path,
                                 flight_data_file_name,
                                 csv_flight_data_file_path,
                                 csv_flight_data_name)

    def setUp(self):
        # Sets up all of the test variables and locations to be used
        # throughout.
        self.base_path = os.path.join(os.path.dirname(__file__),
                                      "test_files") + os.sep
        self.base_path = self.base_path.replace(os.sep, "/")
        # Imports data from template.
        with open(self.base_path + 'test_Input_File.json') as file:
            self.data = json.load(file)
        self.flight_number = self.data["log_to_xlsx_input"]["flight_number"].replace(" ", "").split(",")
        self.flight_date = self.data["log_to_xlsx_input"]["date"].replace(" ", "").split(",")
        self.template_file_path = self.base_path
        self.template_file_name = self.data["flight_log_generator_input"][
            "template_file_name"]
        self.flight_log_file_path = self.base_path + self.data[
            "flight_log_generator_input"]["flight_log_destination"]
        self.flight_data_file_path = self.base_path  # Path to xlsx file
        self.flight_data_file_name = "test_xlsx.xlsx"
        self.flight_data_file_name_list = ["test_xlsx.xlsx"]
        self.csv_flight_data_file_path = self.base_path + self.data[
            "flight_log_generator_input"]["csv_flight_data_file_path"]
        self.csv_flight_data_name = self.data["flight_log_generator_input"
        ]["csv_flight_data_name"].replace(" ", "").split(",")
        self.flight_log_file_name_header = "test_generated_flight_log"
        self.start_time_hours = self.data["flight_log_generator_input"][
            "start_time_hours"].replace(" ", "").split(",")
        self.end_time_hours = self.data["flight_log_generator_input"][
            "end_time_hours"].replace(" ", "").split(",")
        self.metar_file_path = self.base_path + self.data[
            "flight_log_generator_input"]["metar_file_path"]
        self.comp_data_file_path = self.flight_data_file_path + "test_xlsx.pkl"
        self.weather_data = flight_log_code.multi_dictionary_data_formatter(self.data["weather_data"], self.flight_date
                                                                            , "weather_data")
        self.weather_data_dict = self.data["weather_data"]
        self.runway_data = flight_log_code.multi_dictionary_data_formatter(self.data["runway_data"], self.flight_date,
                                                                           "runway_data")
        self.runway_data_dict = self.data["runway_data"]

    def tearDown(self):
        # Removes generated flight log.
        base_path = os.path.join(os.path.dirname(__file__),
                                 "test_files") + os.sep
        generated_file_names = [base_path + "test_generated_flight_logtest_xlsx.ipynb", base_path +
                               "test_generated_flight_log20190123_2.ipynb"]
        for item in range(len(generated_file_names)):
            if os.path.exists(generated_file_names[item]):
                os.remove(generated_file_names[item])

        # Removes pickled data.
        pickle_file_names = [base_path + "test_xlsx.pkl"]
        for item in range(len(pickle_file_names)):
            if os.path.exists(pickle_file_names[item]):
                os.remove(pickle_file_names[item])

    def test_flight_log_maker(self):
        self.ICAO_airfield = nearest_ICAO_finder.icao_finder(
            self.flight_data_file_path,
            self.flight_data_file_name)
        flight_log_code.flight_log_maker(self.template_file_path,
                                         self.template_file_name,
                                         self.flight_log_file_path,
                                         self.flight_data_file_path,
                                         self.flight_data_file_name_list,
                                         self.csv_flight_data_file_path,
                                         self.csv_flight_data_name,
                                         self.flight_date,
                                         self.flight_number,
                                         self.flight_log_file_name_header,
                                         self.ICAO_airfield,
                                         self.start_time_hours,
                                         self.end_time_hours,
                                         self.metar_file_path,
                                         self.weather_data,
                                         self.runway_data,
                                         True)
        # This code tests the flight_log_maker function.
        # First, check that a file has been created.
        test_flight_log_file_path = self.base_path + 'test_generated_flight_logtest_xlsx.ipynb'
        if os.path.exists(test_flight_log_file_path) is True:
            file_exists = True
        else:
            file_exists = False
        self.assertTrue(file_exists)
        # Checks that the file was recently created.
        filetime = os.stat(test_flight_log_file_path)
        # Finds the time since the file has been created.
        time_diff = datetime.now() - datetime.fromtimestamp(filetime.st_mtime)
        # Checks that the time difference is less than 1 second.
        self.assertLess(time_diff.microseconds, 1e6)
        # Check that the files weren't created in the past.
        self.assertGreaterEqual(time_diff.microseconds, 0)

    def test_flight_data(self):
        # Tests the flight data code.
        frame_list = flight_log_code.flight_data(
            self.flight_data_file_path,
            self.flight_data_file_name)
        # Checks that the expected frame dimensions are the correct size.
        frame_dimensions = [17780, 95220, 22860, 25400, 57141, 44436, 57132,
                            19044, 6, 4, 8]
        if len(frame_list) == 11:
            for frame in range(len(frame_list)):
                self.assertEqual(frame_list[frame].size,
                                 frame_dimensions[frame])
        else:
            # Raises a fault if the length of the frame_list is not as
            # expected.
            raise ValueError('Frame list length is not as expected: {0} != 11'
                             .format(len(frame_list)))

    def test_flight_log_creator(self):
        # Generates content
        content = flight_log_code.contents_opener(self.template_file_path,
                                                  self.template_file_name)
        file_name_data = str(self.flight_date[0]) + "_" + str(self.flight_number[0])
        # Runs the flight log creator code
        flight_log_code.flight_log_creator(content, self.base_path,
                                           file_name_data,
                                           self.flight_log_file_name_header)
        # Checks the file was created correctly
        file_exists = os.path.exists(self.base_path
                                     + 'test_generated_flight_log20190123'
                                       '_2.ipynb')
        self.assertTrue(file_exists)

    def test_date_and_flight_number(self):
        # Gets run in the Jupyter Notebook.
        # Extracts the desired variables from the sample notebook
        date_and_flight_number = notebook_results[1]
        # Checks that the output is as expected
        expected_date_and_flight_number = ('20190123', '2')
        self.assertEqual(expected_date_and_flight_number,
                         date_and_flight_number)

    def test_csv_frame(self):
        # Gets run in the Jupyter Notebook.
        # Runs the sample notebook code to extract teh desired variable
        csv_frame = notebook_results[2]
        try:
            # Checks if the key is correct
            csv_frame['X']
            # Failed if no error is raised
            self.assertFalse(True)
        except KeyError:
            # Passes if an error is raised
            self.assertTrue(True)
        # Loads values
        values = csv_frame['Temp0_degC_ArduinoMicro_20190123_Flight2'
        ]
        # Compares expected values with values from the data frame
        expected_values = [24.38, 24.38, 24.38, 24.44, 25.31, 26.69, 27.69,
                           28.44, 29.00, 29.38, 29.69, 30.00, 30.06, 29.69,
                           29.38, 29.06, 28.81, 28.63, 28.38, 28.25, 28.06,
                           27.88, 27.75, 27.63, 27.50, 27.38, 27.25]
        for item in range(len(expected_values)):
            self.assertEqual(expected_values[item], values[item])

    def test_flight_data_time_sorter(self):
        # Gets run in the Jupyter Notebook.
        data_frames = notebook_results[3]
        # Checks that the frames were split into the correct number of
        # subframes
        self.assertEqual(len(data_frames), 12)
        # Assigns expected titles
        data_frame_section_titles = [
            'Status_unavailable_GPS_20190123_Flight2',
            'aileron_CH1_us_RCIN_20190123_Flight2',
            'Altitude_m_BARO_20190123_Flight2',
            'Airspeed_mpers_ARSP_20190123_Flight2',
            'Desired_Roll_degrees_ATT_20190123_Flight2',
            'VibeX_mperspers_VIBE_20190123_Flight2',
            'NavRoll_unavailable_CTUN_20190123_Flight2',
            'AOA_degrees_AOA_20190123_Flight2',
            'Action_time_hh:mm_WEATHER_20190123_Flight2',
            'take_off_direction_RUNWAY_20190123_Flight2',
            'm_empty_kg_AIRCRAFT_20190123_Flight2',
            'Temp0_degC_ArduinoMicro_20190123_Flight2']
        correct_titles = 0
        # Runs through titles. Checks that each title can be called and adds 1
        # to the counter (correct_titles). If a title is not correct/cannot be
        # called, raise a failure.
        for item in range(len(data_frame_section_titles)):
            try:
                data_frames[item][data_frame_section_titles[item]]
                correct_titles += 1
            except KeyError:
                self.assertEqual(1, 2)
        # Checks that all titles are correct
        self.assertEqual(correct_titles, 12)

    def test_flight_data_and_axis(self):
        # Gets run in the Jupyter Notebook.
        # Loads the data generated previously
        flight_data_and_axis = notebook_results[4]
        # Assigns expected titles
        expected_titles = ['GPS', 'RCIN', 'BARO', 'ARSP', 'ATT', 'VIBE',
                           'CTUN', 'AOA', 'WEATHER', 'RUNWAY', 'AIRCRAFT', 'ArduinoMicro']
        # Checks that the titles are present and in the correct place
        for item in range(len(expected_titles)):
            self.assertEqual(expected_titles[item], flight_data_and_axis[1][item][0])

    def test_file_type_finder(self):
        # Detects xlsx files from the test_files folder
        detected_files = flight_log_code.file_type_finder(self.base_path, ".xlsx")
        expected_file_names = ['test_xlsx.xlsx']
        # Checks that all the expected files are present
        for file_name in expected_file_names:
            if file_name in detected_files:
                file_detected = True
            else:
                file_detected = False
            self.assertTrue(file_detected)

    def test_compile_and_compress(self):
        # Runs the compiler and compressor.
        flight_log_code.compile_and_compress(self.flight_data_file_path, [self.flight_data_file_name],
                                             self.csv_flight_data_file_path, self.csv_flight_data_name,
                                             self.comp_data_file_path)
        pickle_file_name = self.base_path + "test_xlsx.pkl"  # Defines the file name
        # Checks that the file exists
        self.assertTrue(os.path.exists(pickle_file_name))

    def test_dictionary_reader(self):
        # Returns a string of formatted weather data
        read_data = flight_log_code.dictionary_reader(self.weather_data_dict, units_present=True, flight_number="1")
        # Adds a string of formatted runway data
        read_data += flight_log_code.dictionary_reader(self.runway_data_dict, units_present=False, flight_number="2")
        expected_results = [" | Flight | Action time  (hh:mm) | Pressure  (Pa) | Temperature  (C) | Wind direction"
                            "  (degrees) | Wind speed  (mps) |",
                            " | --- | --- | --- | --- | --- | --- |",
                            "| Flight 1| 12:34 | 1234567 | -12.3 | 123 | 12.3 |",
                            " | Flight | runway surface  | surface condition  | take off direction  |",
                            " | --- | --- | --- | --- |",
                            "| Flight 2| XYZ | ZYX | 321 |"]
        # Checks that the correct data is present in the read_data string
        for string in expected_results:
            self.assertTrue(string in read_data)

    def test_multi_string_data_formatter(self):
        """A method to test the multi_string_data_formatter()"""
        # Assigns testing strings
        input1 = "Fish,Ca t,Dog,35, Mouse,   89"
        flight_names_1 = [1, 2, 3, 4, 5, 6]
        debug_name_1 = "Input 1"
        input2 = ""
        flight_names_2 = [0, 1]
        debug_name_2 = "Input 2"
        input3 = "Fail1"
        flight_names_3 = [7, 8]
        debug_name_3 = "TEST"
        # Assigns testing outputs
        expected_output1 = ["Fish", "Cat", "Dog", "35", "Mouse", "89"]
        expected_output2 = [""]
        expected_output3 = False
        # Tests that the method works as expected
        self.assertEqual(flight_log_code.multi_string_data_formatter(input1, flight_names_1, debug_name_1),
                         expected_output1)
        self.assertEqual(flight_log_code.multi_string_data_formatter(input2, flight_names_2, debug_name_2),
                         expected_output2)
        try:
            flight_log_code.multi_string_data_formatter(input3, flight_names_3, debug_name_3)
            # Should raise an index error. If not, fail the test.
            self.assertTrue(expected_output3)
        except IndexError:
            # Planned error
            self.assertTrue(True)

    def test_remove_dictionary_key(self):
        """A method to test the remove_dictionary_key()"""
        # Create variables
        dictionary1 = {"DOG": "4F", "Mouse": "Cat", "Donkey": 7, 8906: "Fish", "PiG": "Farm"}
        keys_to_remove = ["Mouse", 8906, "DOG"]
        # Remove the keys and check if they have been removed
        for key_index in range(len(keys_to_remove)):
            output = flight_log_code.remove_dictionary_key(dictionary1, keys_to_remove[key_index])
            if keys_to_remove[key_index] in output.keys():
                # Fail if the key is still present
                self.assertTrue(False)
            else:
                # Key successfully removed, pass the test
                self.assertTrue(True)

    def test_multi_dictionary_data_formatter(self):
        """A method to test the multi_dictionary_data_formatter()"""
        # Create variables
        dictionary1 = {"DOG": "4F, Squirrel", "Mouse": "Cat, 78", "Donkey": "7, 7", 8906: "Fish , Bird",
                       "PiG": "Farm, Animal"}
        debug_name1 = "dict1"
        flight_dates1 = [0, 1]
        dictionary2 = {"Face": "Nose, Ear, Mouth"}
        flight_dates2 = [2, 3]
        debug_name2 = "TEST"

        # Assigns expected outputs:
        expected_output_dict1_1 = ["4F", "Cat", "7", "Fish", "Farm"]
        expected_output_dict1_2 = ["Squirrel", "78", "7", "Bird", "Animal"]
        key = ["DOG", "Mouse", "Donkey", 8906, "PiG"]
        expected_output_dict2 = False

        # Runs the method
        output1 = flight_log_code.multi_dictionary_data_formatter(dictionary1, flight_dates1, debug_name1)

        try:
            flight_log_code.multi_dictionary_data_formatter(dictionary2, flight_dates2, debug_name2)
            # Fails if this does not raise an error
            self.assertTrue(expected_output_dict2)
        except IndexError:
            # Passes if an error is raised
            self.assertFalse(expected_output_dict2)

        for key_index in range(len(dictionary1.keys())):
            self.assertEqual(output1[0][key[key_index]], expected_output_dict1_1[key_index])
            self.assertEqual(output1[1][key[key_index]], expected_output_dict1_2[key_index])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Unittests for the nearest_ICAO_finder code.

@author: Adrian Weishaeupl
aw6g15@soton.ac.uk 2019
"""

from autoflpy.util import nearest_ICAO_finder
import unittest
import numpy as np
import os


class TestNearestICAOFinder(unittest.TestCase):

    def setUp(self):
        # Defines variables needed
        # Set the working directory
        base_path = os.path.join(os.path.dirname(__file__),
                                 "test_files") + os.sep
        # Tidies up the base path for python.
        self.base_path = base_path.replace(os.sep, "/")
        # Define the variables
        self.excel_file_path = self.base_path
        self.excel_file_name = "test_xlsx.xlsx"

    def test_icao_finder(self):
        # Runs icao_finder function
        icao = nearest_ICAO_finder.icao_finder(self.excel_file_path,
                                               self.excel_file_name)
        # Checks that the correct icao code is returned.
        self.assertEqual('EGHE', icao)

    def test_closest_icao(self):
        # Runs the airport_lat_long_clst function
        airport_lat_long = nearest_ICAO_finder.airport_lat_long()
        # Arbitrary latitude and longitude for checking
        uav_lat_long_check = [[[51.067397], [-1.320574]],
                              [[50.154234], [8.735601]],
                              [[49.721109], [-102.279640]],
                              [[-40.424322], [-68.260537]],
                              [[-8.266594], [28.914712]]]
        # Defines the expected outputs
        expected_closest_icao = [14637, 14122, 12234, 30551, 16258]
        # Runs closest_icao function from the nearest_ICAO_finder.py
        for lat_long in range(len(uav_lat_long_check)):
            result = nearest_ICAO_finder.closest_icao(np.array(
                    uav_lat_long_check[lat_long]), np.array(
                            [airport_lat_long[:, 1], airport_lat_long[:, 2]]))
            # Check the results are equal to the expected
            self.assertEqual(result, expected_closest_icao[lat_long])

    def test_airport_lat_long(self):
        # Runs the airport_lat_long_clst function
        airport_lat_long = nearest_ICAO_finder.airport_lat_long()
        # Creates lists of items and expected results
        item_check = [10, 100, 1000, 10000, 20000, 30000, 40488]
        expected_icaos = ['00FL', '01OI', '0PA1', '9MI6', 'KDGL', 'RJNO',
                          'ZZZZ']
        for item_index in range(len(item_check)):
            # Finds ICAO
            icao = airport_lat_long[item_check[item_index]][0]
            # Compares ICAO to the expected index
            expected_icao = expected_icaos[item_index]
            self.assertEqual(icao, expected_icao)

    def test_uav_lat_long(self):
        # Runs the uav_lat_long_clst function from the nearest_ICAO_finder.py
        uav_lat_long = nearest_ICAO_finder.uav_lat_long(self.excel_file_path,
                                                        self.excel_file_name)
        # Defines expected_results
        expected_lat_long = np.array([[49.9543952], [-6.3688301]])
        # Compares the uav_lat_long_clst to the expected_lat_long, 0 = lat, 1 = long
        self.assertEqual(expected_lat_long[0], uav_lat_long[0])
        self.assertEqual(expected_lat_long[1], uav_lat_long[1])

    def test_multi_icao_finder(self):
        """Tests the multi_icao_finder()"""
        # Assigns variables
        file_path = self.base_path
        file_names = [self.excel_file_name, self.excel_file_name]
        flight_names = ["test_xlsx.xlsx", "test_xlsx.xlsx"]

        # Runs the method
        icaos = nearest_ICAO_finder.multi_icao_finder(file_path, file_names, flight_names)

        # Tests the results
        expected_icaos = ["EGHE", "EGHE"]
        for item in range(len(icaos)):
            self.assertEqual(icaos[item], expected_icaos[item])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Unit tests for the plotting.py code.
No tests are currently written for the main plotting methods.

@author Adrian Weishaeupl (aw6g15@soton.ac.uk)
"""

from autoflpy.util import plotting
import unittest
import os
import pickle as pk


class TestPlotting(unittest.TestCase):

    def setUp(self):
        # Create variables and assign directories before any test.
        base_path = os.path.join(os.path.dirname(__file__),
                                 "test_files") + os.sep
        # Tidies up the base path for python.
        self.base_path = base_path.replace(os.sep, "/")

        self.values_list = pk.load(open(base_path + "test_pickled_data.pkl", "rb"))
        self.values_list_multi = pk.load(open(base_path + "test_pickled_data_multi.pkl", "rb"))

    def test_manual_time_offset(self):
        """Test for manual_time_offset()"""
        # Defining inputs
        time_x_offset = [1234]

        # Changes the values list to only include the data
        plot_data, flight_dates_list, single_flight, number_of_flights =\
            plotting.single_flight_detection(self.values_list)

        # Running the method
        values_list_offset = plotting.manual_time_offset(plot_data, time_x_offset, number_of_flights)

        # Loads "clean" values_list data - plotting.manual_time_offset() seems to add the offset to all time data
        # - no matter what variable it is in. Known to affect self.values_list, plot_data and values_list_offset
        values_list = pk.load(open(self.base_path + "test_pickled_data.pkl", "rb"))

        # Testing the result
        # time_data_gps = values_list[0][1][0][14][2]
        # time_data_vibe = values_list[0][1][6][8][2]
        for data_point in range(len(values_list[0][1][0][14][2])):

            # Checks that the correct amount has been added to the time column
            self.assertEqual((values_list[0][1][0][14][2][data_point] + time_x_offset[0]),
                             values_list_offset[0][0][14][2][data_point])

        for data_point in range(len(values_list[0][1][6][7][2])):
            # Checks that the correct amount has been added to the time column
            self.assertEqual((values_list[0][1][6][7][2][data_point] + time_x_offset[0]),
                             values_list_offset[0][6][7][2][data_point])

    def test_select_plot_data_single(self):
        """Tests for select_plot_data_single()"""
        # Assign variables
        plot_information = [["x", "time", "gps"], ["y", "altitude", "gps"]]
        print("TEST START")
        # Changes the values list to only include the data
        values_list, flight_dates_list, single_flight, number_of_flights =\
            plotting.single_flight_detection(self.values_list)

        # Runs the method
        plot_data = plotting.select_plot_data_single(values_list, plot_information, number_of_flights)

        # Checks that the called data is correct
        for data_index in range(len(values_list[0][0][14][2])):
            # time_data_gps = values_list[0][0][14][2]
            self.assertEqual(values_list[0][0][14][2][data_index], plot_data[0][0][1][2][data_index])
            # altitude_data_gps = values_list[0][0][8][2]
            self.assertEqual(values_list[0][0][8][2][data_index], plot_data[0][1][1][2][data_index])

    def test_arm_data_formatting(self):
        """Test for arm_data_plotting()"""
        # Assigns variables to be testes
        arm_data1 = False
        arm_data2 = True

        # Changes the values list to only include the data
        values_list, flight_dates_list, single_flight, number_of_flights = \
            plotting.single_flight_detection(self.values_list)

        # Runs the arm_data_formatting method
        output1_0, output1_1 = plotting.arm_data_formatting(arm_data1, values_list, number_of_flights, flight_dates_list)
        output2_0, output2_1 = plotting.arm_data_formatting(arm_data2, values_list, number_of_flights, flight_dates_list)

        # Checks that the outputs are correct
        self.assertEqual(False, output1_0)
        self.assertEqual(None, output1_1)
        self.assertEqual(True, output2_0)
        self.assertEqual([[['', ['Id', 'unavailable', [10, 11]]], ['', ['Time', 's', [26.894238, 264.634104]]]]],
                         output2_1)

    def test_single_flight_detection(self):
        """Tests for single_flight_detection()"""
        # TODO: Test that the values lists are formatted correctly as well

        # Runs the method
        values_list1, flight_data_list1, single_flight1, number_of_flights1 = \
            plotting.single_flight_detection(self.values_list)
        values_list2, flight_data_list2, single_flight2, number_of_flights2 = \
            plotting.single_flight_detection(self.values_list_multi)

        self.assertTrue(single_flight1)
        self.assertFalse(single_flight2)
        self.assertEqual(1, number_of_flights1)
        self.assertEqual(2, number_of_flights2)
        self.assertEqual([''], flight_data_list1)
        self.assertEqual(['20190110_Flight1', '20190110_Flight2'], flight_data_list2)


if __name__ == '__main__':
    unittest.main()




import os
from openpyxl import Workbook
from openpyxl.cell import WriteOnlyCell



"""
This code converts a .log file (generated using mission planner from a .bin
file) into a .xls document to be used with the automated flight log creator.
@author: Adrian Weishaeupl
aw6g15@soton.ac.uk 2019

Based on work done by Samuel Pearson (sp1g18@soton.ac.uk) (06-08/2019)
"""


def log_reader(log_file_path, name_converter_file_path, data_sources_path,
               excel_file_path, excel_file_name, flight_date, flight_number, weather_data, runway_data, aircraft_data):
    """Creates a formatted excel file from a log file. """
    print('Starting log reader')
    print('Creating new work book')
    # Creates a new write only workbook for faster writing
    workbook = Workbook(write_only=True)
    # Opens log file
    print('Reading log file')
    log_opened = open(log_file_path, "r")
    # Reads contents
    log_contents_text = log_opened.read()
    # Closes file
    log_opened.close()
    # Opens file
    name_list_opened = open(name_converter_file_path, "r")
    # Reads contents
    name_list_text = name_list_opened.read()
    # Closes file
    name_list_opened.close()
    # Opens file
    data_sources_opened = open(data_sources_path, "r")
    # Reads contents
    data_sources_text = data_sources_opened.read()
    # Closes file
    data_sources_opened.close()
    # Splits text from data sources into individual lines
    data_sources = data_sources_text.split("\n")[1:]
    # splits name list into lines and ignored the first key line.
    name_list = name_list_text.split("\n")[1:]
    # splits log contents about each new line
    log_contents = log_contents_text.split("\n")

    # Goes through each line
    print('Populating work book')
    for line in log_contents:
        # Splits data into columns
        data = line.split(", ")
        if data[0] == "FMT":
            # Specifies whether data is available
            data_available = False
            # Checks to see if data was recorded for a particular heading.
            for check_line in log_contents:
                # splits data into lines
                check_line_list = check_line.split(", ")
                # Checks through all data to see if there was any data recorded
                # for a particular variable.
                if data[3] == check_line_list[0]:
                    # Specifies that data is available
                    data_available = True
                    # breaks from for loop
                    break
            if data[3] != "FMT" and data[3] != "UNIT" and data[3] != "FMTU" \
                    and data_available is True and data[3] in data_sources:

                # Defines line for titles and resets this for every sheet
                heading_line = []

                # Creates a new worksheet for all of the data.
                worksheet = workbook.create_sheet(title=data[3])

                # Excludes the first time column and puts it at the end.
                data_list_time_end = data[-1].split(",")[1:]
                # Appends time column at the end.
                data_list_time_end.append(data[-1].split(",")[0])
                # Creates the headings and appends the time column to the end
                # to match the format of the previous data sets.
                for heading_name in data_list_time_end:
                    # Code will be here to find the units
                    unit = "unavailable_"
                    # heading name check code will go here.
                    heading = heading_name
                    # Goes through the names in the name_list to check the
                    # units
                    for name_data in name_list:
                        # splits name list
                        name_info = name_data.split(", ")
                        # Checks to see if the information in the list matches
                        # that being from the log file
                        if name_info[0] == data[3] and name_info[1] == \
                                heading_name:
                            # Sets heading name to be same as from name
                            heading = name_info[2]
                            # Checks to see if there were units
                            if unit == "no unit":
                                unit = ""
                            else:
                                # Sets unit to be that from name converter list
                                unit = name_info[3] + "_"
                            break
                    # Creates heading from data
                    heading = heading + "_" + unit + data[3] + "_" + flight_date + "_Flight" + flight_number

                    # Creates the heading line
                    heading_cell = WriteOnlyCell(worksheet, value=heading)
                    heading_line.append(heading_cell)

                # Writes heading line to the worksheet
                worksheet.append(heading_line)

                # Goes through all data searching for a match
                for lines in log_contents:
                    # Splits line data
                    line_data = lines.split(", ")
                    #  Checks to see if data name is the one being searched.
                    if line_data[0] == data[3]:
                        # Resets row to be written
                        row = []
                        # Goes through all data in line, starting from the
                        # column after the time column
                        for recorded_data in line_data[2:]:
                            row.append(recorded_data)
                        # Appends time data for the row
                        row.append(line_data[1])
                        # Writes the row to the worksheet
                        worksheet.append(row)
        else:
            # Ends for loop and so saves code
            break

    # Adds custom weather data to the xlsx document
    worksheet = workbook.create_sheet("WEATHER_DATA")
    weather_keys_to_be_named = list(weather_data.keys())
    weather_keys_to_be_named.append("dummy_time")  # To sort the runway data in flight_data_time_sorter()
    # Formats names correctly
    weather_keys = []
    for key in weather_keys_to_be_named:
        weather_keys.append(str(key) + "_WEATHER_" + str(flight_date) + "_Flight" + str(flight_number))
    weather_values = list(weather_data.values())
    weather_values.append("N/A")
    worksheet.append(weather_keys)
    worksheet.append(weather_values)

    # Adds custom runway data to the xlsx document
    worksheet = workbook.create_sheet("RUNWAY_DATA")
    runway_keys_to_be_named = list(runway_data.keys())
    runway_keys_to_be_named.append("dummy_time")  # To sort the runway data in flight_data_time_sorter()
    # Formats names correctly
    runway_keys = []
    for key in runway_keys_to_be_named:
        runway_keys.append(str(key) + "_RUNWAY_" + str(flight_date) + "_Flight" + str(flight_number))
    runway_values = list(runway_data.values())
    runway_values.append("N/A")
    worksheet.append(runway_keys)
    worksheet.append(runway_values)

    # Adds custom aircraft data to the xlsx document
    worksheet = workbook.create_sheet("AIRCRAFT_DATA")
    aircraft_keys_to_be_named = list(aircraft_data.keys())
    aircraft_keys_to_be_named.append("dummy_time")  # To sort the runway data in flight_data_time_sorter()
    # Formats names correctly
    aircraft_keys = []
    for key in aircraft_keys_to_be_named:
        aircraft_keys.append(str(key) + "_AIRCRAFT_" + str(flight_date) + "_Flight" + str(flight_number))
    aircraft_values = list(aircraft_data.values())
    aircraft_values.append("N/A")
    worksheet.append(aircraft_keys)
    worksheet.append(aircraft_values)

    # Saves file
    print('Saving workbook')
    xlsx_file_name_and_path = excel_file_path + os.sep + excel_file_name + ".xlsx"
    workbook.save(filename=xlsx_file_name_and_path)
    print('Log reader finished for {}'.format(str(excel_file_name)))


def log_reader_multi(log_file_paths, name_converter_file_path, data_sources_path,
                     excel_file_path, excel_file_names, flight_dates, flight_numbers, weather_data_multi,
                     runway_data_multi, aircraft_data_multi):
    """Runs the log_reader for once per flight log entered in the Input_file.json"""
    # Iterates through the number of flights
    for flight in range(len(flight_numbers)):
        print("Creating workbook for {}".format(str(excel_file_names[flight])))
        log_reader(log_file_paths[flight], name_converter_file_path, data_sources_path, excel_file_path,
                   excel_file_names[flight], flight_dates[flight], flight_numbers[flight], weather_data_multi[flight],
                   runway_data_multi[flight], aircraft_data_multi[flight])



"""
This code processes weather data.

@author Adrian Weishaeupl
aw6g15@soton.ac.uk 2019

"""

import pandas as pd
import scipy.constants as co
from ADRpy import unitconversions as uc


def environmental_data_filtering(values_list):
    """A method for returning the environmental data from a values_list data set
    returns data in a list of [weather_data, runway_data, flight] entries"""
    data_set = []
    number_of_flights = len(values_list)
    for flight in range(number_of_flights):
        values_list_dataframe = pd.DataFrame(values_list[flight][1])
        index = values_list_dataframe[0]  # Creates a set of titles to be written over the titles present
        transposed_values_list_dataframe = values_list_dataframe.transpose()[1:]  # Removes the old titles
        data_only = transposed_values_list_dataframe.transpose()  # Turns the data frame to index the columns
        # Indexes the columns and turns the data frame so that they can be called:
        data_with_index = data_only.set_index(index).transpose()
        # Calls the data
        runway_data = list(data_with_index["RUNWAY"])
        weather_data = list(data_with_index["WEATHER"])
        aircraft_data = list(data_with_index["AIRCRAFT"])
        data_set.append([weather_data, runway_data, aircraft_data, values_list[flight][0]])
    return data_set


def environmental_data_dictionary(values_list, index):
    """Designed for the weather data (index = 0) and runway data (index = 1)

    Returns a dictionary data set in the form:
    [[dictionary_1, flight_1], [dictionary_2, flight_2]]
    """
    data_set = environmental_data_filtering(values_list)
    data_list = []
    number_of_flights = len(values_list)
    # Runs through the number of flights and creates a dictionary for each flight
    for flight in range(number_of_flights):
        data = {}
        data_raw = data_set[flight][index]
        for entry in data_raw:
            if entry is not None and "dummy" not in entry[0].lower():
                name = str(entry[0]) + "_" + str(entry[1])
                name = name.replace(" ", "_")
                data_value = entry[-1][0]  # the [0] is to return just the element (not a list)
                data[name] = data_value
        if index == 0:
            # Additional data calculations for weather data
            data["Density_kgperm3"] = data["Pressure_Pa"]/(287. * uc.c2k(data["Temperature_C"]))
        data_list.append([data, values_list[flight][0]])
    return data_list


def weather_data_dictionary(values_list):
    """Creates a list of dictionary of weather_data from a values_list. One dictionary is created per flight

    returns
        [[weather_dictionary_1, flight_1], [weather_dictionary_2, flight_2]]
        where weather_dictionary is a dictionary of weather data"""
    return environmental_data_dictionary(values_list, 0)


def runway_data_dictionary(values_list):
    """Creates a list of dictionary of runway_data from a values_list. One dictionary is created per flight

    returns
        [[runway_dictionary_1, flight_1], [runway_dictionary_2, flight_2]]
        where runway_dictionary is a dictionary of runway data"""
    return environmental_data_dictionary(values_list, 1)


def aircraft_data_dictionary(values_list):
    """Creates a list of dictionary of aircraft_data from a values_list. One dictionary is created per flight

    returns
        [[aircraft_dictionary_1, flight_1], [aircraft_dictionary_2, flight_2]]
        where aircraft_dictionary is a dictionary of aircraft data"""
    return environmental_data_dictionary(values_list, 2)



"""
This modules allows for the running of the log to xlsx converter and the
automatic flight log generator.
"""

__author__ = "Adrian Weishaeupl"

import autoflpy.util.name_generator as name_generator
import os
import json
import autoflpy.util.flight_log_code as flight_log_code
import autoflpy.util.log_to_xlsx as log_to_xlsx
import autoflpy.util.nearest_ICAO_finder as nearest_ICAO_finder
from shutil import copyfile

"""
TODO:
    Calculate wind speed/vector/plot.
    Add flight duration from vibration data.
    Add a 3D map image to summarise flight.
    Option to display time in UTC or time after flight start.
"""


def autoflpy(input_file='Input_File.json', include_metar=False, run_log_to_xlsx=True):
    """autoflpy serves as the user interface for flight report generation.

    Upon first time running of the script, sample data will be used to create a sample flight report for the user to
    look at for inspiration.

    A folder structure containing the "Input_File.json" and base file paths will also be created.
    The user can then complete the "Input_File.json" with appropriate information and analyse their own flight data.

    Folder structure generated:

    user_files
        * csv_data                  Contains sample CSV data. User data in the form of a csv file can be added here.
        * excel_file_path		    Contains excel files generated from the log files.
        * flight_logs_generated		Contains the generated flight reports.
        * log_files					Contains the user input flight data in the .log format.
        * METAR_storage				This acts as a database for the METAR data.

    arguments:
        include_metar=False
            If set to True, METAR information from the nearest airport will be collected and added to the log. This only
            works when data from a single flight is being analysed.

        run_log_to_xlsx=True
            If set to False, xlsx files will not be generated which will save time. Do this if these files have already
            been created. It will cause errors if the xlsx files are not present or in the wrong directory.
    """
    # Finds the file path from where this code is being run.
    base_path = os.path.join(os.path.dirname(__file__), "data") + os.sep
    # Tidies up the base path for python.
    base_path = base_path

    # Sets the default storage location in the working directory.
    default_storage_path = str((os.getcwd() + os.sep + 'user_files' + os.sep
                                ))

    # Copies the standard input file into the working directory
    if os.path.exists(default_storage_path + input_file) is False:
        os.makedirs(default_storage_path, exist_ok=True)
        print('Storage directory made in: {}'.format(default_storage_path))
        copyfile(base_path + input_file, default_storage_path + input_file)
    else:
        pass

    data_sources_path = default_storage_path + 'Data_sources.txt'
    if os.path.exists(data_sources_path) is False:
        copyfile(base_path + 'Data_sources.txt', data_sources_path)
    else:
        pass
    # Copies the template input file into the current working directory
    input_template_file = 'Input_File_Template.json'
    if os.path.exists(default_storage_path + input_template_file) is False:
        copyfile(base_path + input_template_file, default_storage_path +
                 input_template_file)
    else:
        pass

    # Reads the test_input_file information.
    with open(default_storage_path + input_file) as file:
        data = json.load(file)

    # Sets  variables from the input file to be used.
    # If no log file path has been entered, go to the standard log path.
    log_file_paths = []
    if data["log_to_xlsx_input"]["log_file_path"] != "" and os.path.exists(
      data["log_to_xlsx_input"]["log_file_path"]) is True:
        for file in data["log_to_xlsx_input"]["log_file_name"].replace(" ", "").split(","):
            log_file_paths.append(data["log_to_xlsx_input"][
                                      "log_file_path"] + os.sep + file)
    else:
        # Creates a new directory to look for files.
        log_file_base_path = (default_storage_path + "log_files" + os.sep
                              )
        for file in data["log_to_xlsx_input"]["log_file_name"].replace(" ", "").split(","):
            log_file_paths.append(log_file_base_path + os.sep + file)
        try:
            os.makedirs(log_file_base_path)
            # Raises error and gives advice on how to continue.
            raise FileNotFoundError('No log file directory was entered. A new directory has been'
                                    ' made.')
        except OSError:
            print('Log file path found.')
            # Copies the example data file into the log storage
            copyfile(base_path + 'test_log_to_xlsx.log', log_file_base_path +
                     'test_log_to_xlsx.log')

    name_converter_file_path = base_path + 'Name_converter_list.txt'
    # If no excel data file path has been entered, go to the standard path.
    if data["log_to_xlsx_input"]["excel_data_file_path"] != "" and \
        os.path.exists(
            data["log_to_xlsx_input"]["excel_data_file_path"]) is True:
        excel_file_path = (data["log_to_xlsx_input"][
            "excel_data_file_path"])
    else:
        # Makes a directory in the current working path to be used.
        excel_file_path = default_storage_path + "excel_file_path"
        try:
            os.makedirs(excel_file_path)
        except OSError:
            print("Excel folder found. Will use this folder to store generated"
                  " xlsx files.")
    # Formats the flight dates and flight numbers as lists and checks their length against the length of flight_dates
    flight_dates = str(data["log_to_xlsx_input"]["date"]).replace(" ", "").split(",")
    flight_numbers = flight_log_code.multi_string_data_formatter(data["log_to_xlsx_input"]["flight_number"],
                                                                 flight_dates, "flight_numbers")
    # Generates an appropriate file name
    excel_file_names = name_generator.excel_file_name_updater(flight_dates, flight_numbers)

    # Imports custom weather data entered into the input file
    try:
        weather_data_multi = data["weather_data"]
    except KeyError:
        print("'weather_data' has not been entered.")
        weather_data_multi = {}

    # Imports custom runway data entered into the input file
    try:
        runway_data_multi = data["runway_data"]
    except KeyError:
        print("'runway_data' has not been entered.")
        runway_data_multi = {}

    # Imports custom aircraft data entered in the input file
    try:
        aircraft_data_multi = data["aircraft_data"]
    except KeyError:
        print("'aircraft_data' has not been entered.")
        aircraft_data_multi = {}

    # Formats the weather data into several separate dictionaries
    weather_data_lists = flight_log_code.multi_dictionary_data_formatter(
        weather_data_multi, flight_dates, "weather_data")
    # Formats the runway data into several separate dictionaries
    runway_data_lists = flight_log_code.multi_dictionary_data_formatter(
        runway_data_multi, flight_dates, "runway_data")
    aircraft_data_lists = flight_log_code.multi_dictionary_data_formatter(
        aircraft_data_multi, flight_dates, "aircraft_data")

    if run_log_to_xlsx is True:
        # Runs the xlsx converter
        log_to_xlsx.log_reader_multi(log_file_paths,
                                     name_converter_file_path,
                                     data_sources_path,
                                     excel_file_path,
                                     excel_file_names,
                                     flight_dates,
                                     flight_numbers,
                                     weather_data_lists,
                                     runway_data_lists,
                                     aircraft_data_lists)
    else:
        print("log_to_xlsx has been disabled. This will cause errors if .xlsx data has not been generated previously or"
              " is not in the correct folder.")

    start_times_hours = flight_log_code.multi_string_data_formatter(
        data["flight_log_generator_input"]["start_time_hours"], flight_dates, "start_time_hours")
    end_times_hours = flight_log_code.multi_string_data_formatter(
        data["flight_log_generator_input"]["end_time_hours"], flight_dates, "end_time_hours")
    csv_flight_data_names = flight_log_code.multi_string_data_formatter(
        data["flight_log_generator_input"]["csv_flight_data_name"], flight_dates, "csv_flight_data_name")
    flight_data_file_names = []
    for name in excel_file_names:
        flight_data_file_names.append(str(name) + ".xlsx")

    # Assigns variables - checks if any information is entered into the input
    # file for the directories before creating new directories in the current
    # working directory.
    if data["flight_log_generator_input"]["template_file_path"] != "" and \
        os.path.exists(
            data["flight_log_generator_input"]["template_file_path"]) is True:
        template_file_path = (data["flight_log_generator_input"][
            "template_file_path"])
    else:
        template_file_path = default_storage_path
    if data["flight_log_generator_input"]["template_file_name"] != "" and \
        os.path.exists(
            template_file_path +
            data["flight_log_generator_input"]["template_file_name"]) is True:
        template_file_name = data["flight_log_generator_input"][
            "template_file_name"]
    else:
        template_file_name = 'Default_Template_(Full_Summary).ipynb'
        print('Using default template')
        # Checks that the file is not already present
        if os.path.exists(default_storage_path + template_file_name) is False:
            # Copies the default template into the default storage directory
            copyfile(base_path + template_file_name, default_storage_path +
                     template_file_name)
    if data["flight_log_generator_input"]["flight_log_destination"] != "" and \
        os.path.exists(
            data["flight_log_generator_input"][
                "flight_log_destination"]) is True:
        flight_log_file_path = data["flight_log_generator_input"][
            "flight_log_destination"]
    else:
        # Makes a directory in the current working path to be used.
        flight_log_file_path = default_storage_path + "flight_logs_generated"
        try:
            os.makedirs(flight_log_file_path)
        except OSError:
            print("Flight log folder found. Will use this folder to store "
                  "generated flight log files.")
    flight_data_file_path = (excel_file_path + os.sep)
    if data["flight_log_generator_input"][
        "csv_flight_data_file_path"] != "" and \
        os.path.exists(data["flight_log_generator_input"]
                       ["csv_flight_data_file_path"]) is True:
        csv_flight_data_file_path = data[
            "flight_log_generator_input"]["csv_flight_data_file_path"]
    else:
        # Makes a directory in the current working path to be used.
        csv_flight_data_file_path = (default_storage_path + "csv_flight_data" + os.sep
                                         )
        try:
            os.makedirs(csv_flight_data_file_path)
        except OSError:
            print('CSV flight data folder found.')
        # Copies the csv test data into the new file path
        copyfile(base_path + 'test_csv.CSV', csv_flight_data_file_path
                 + 'test_csv.CSV')

    flight_log_file_name_header = "Flight_report_"

    if data["flight_log_generator_input"][
        "metar_file_path"] != "" and \
        os.path.exists(
            data["flight_log_generator_input"]["metar_file_path"]) is True:
        metar_file_path = data[
            "flight_log_generator_input"]["metar_file_path"]
    else:
        # Makes a directory in the current working path to be used.
        metar_file_path = (default_storage_path + "METAR_storage" +
                           os.sep)
        try:
            os.makedirs(metar_file_path)
        except OSError:
            print('METAR storage folder found.')
    # Creates images folder for the logo
    images_path = (default_storage_path + 'images' + os.sep
                   )
    try:
        os.makedirs(images_path)
    except OSError:
        print('Images folder found.')
        # Copies the test logo into the new file path if it is not already
        # present
    if os.path.exists(images_path + 'Your_logo_file_name_here.png') is False:
        copyfile(base_path + 'Your_logo_file_name_here.png',
                 images_path + 'Your_logo_file_name_here.png')

    # Excludes METAR for multiple flights
    if include_metar is True:
        if len(flight_dates) > 1:
            include_metar = False
            print("Metar is disabled for multiple flights")
        else:
            include_metar = True

    # Finds the nearest airfield for METAR information
    if include_metar is True:
        icao_airfields = nearest_ICAO_finder.multi_icao_finder(flight_data_file_path,
                                                               flight_data_file_names, excel_file_names)
    else:
        # ICAO airfields not required
        icao_airfields = []

    # Runs the flight log generator
    flight_log_code.flight_log_maker(template_file_path,
                                     template_file_name,
                                     flight_log_file_path,
                                     flight_data_file_path,
                                     flight_data_file_names,
                                     csv_flight_data_file_path,
                                     csv_flight_data_names,
                                     flight_dates,
                                     flight_numbers,
                                     flight_log_file_name_header,
                                     icao_airfields,
                                     start_times_hours,
                                     end_times_hours,
                                     metar_file_path,
                                     weather_data_lists,
                                     runway_data_lists,
                                     include_metar)


if __name__ == "__main__":
    autoflpy(run_log_to_xlsx=True, include_metar=True)



# -*- coding: utf-8 -*-
"""
Unit tests for the take_off_detection module
Created on Tue Feb 18 14:53:20 2020

@author Adrian Weishaeupl (aw6g15@soton.ac.uk)
"""

from autoflpy.util.analysis import take_off_detection
import unittest
import os
import pickle as pk


def load_values_list_from_pickle(data_file_path):
    # Uncompress the data.
    data = open(data_file_path, "rb")
    values_list = pk.load(data)
    data.close()
    return values_list


class TestTakeOffDetection(unittest.TestCase):

    def setUp(self):
        self.base_path = os.path.join(os.path.dirname(__file__),
                                      "test_files") + os.sep
        self.base_path = self.base_path.replace(os.sep, "/")
        self.values_list = load_values_list_from_pickle(self.base_path + "test_pickled_data.pkl")

    def test_take_off_point_finder(self):
        """Tests the take_off_point_finder"""
        # Generates the data
        values_list = self.values_list

        take_off_time_alt_list = []
        take_off_groundspeed_list = []
        take_off_time_spd_list = []
        sensitivity_list = [0.05, 0.1, 0.2, 0.5, 0.7]

        # Runs the code with different sensitivities
        for sensitivity in sensitivity_list:
            take_off_time_alt, take_off_groundspeed, take_off_time_spd = \
                take_off_detection.take_off_point_finder(values_list, sensitivity, sensitivity)
            take_off_time_alt_list.append(take_off_time_alt)
            take_off_groundspeed_list.append(take_off_groundspeed)
            take_off_time_spd_list.append(take_off_time_spd)

        expected_take_off_time_alt_list = [33.21, 33.41, 33.61, 34.41, 34.81]
        expected_take_off_groundspeed_list = [20.25, 21.47, 22.54, 25.42, 26.30]
        expected_take_off_time_spd_list = [30.81, 30.81, 31.01, 31.41, 31.61]

        # Compares the generated values to the expected values
        for item in range(len(sensitivity_list)):
            self.assertEqual(take_off_groundspeed_list[item], expected_take_off_groundspeed_list[item])
            self.assertEqual(take_off_time_alt_list[item], expected_take_off_time_alt_list[item])
            self.assertEqual(take_off_time_spd_list[item], expected_take_off_time_spd_list[item])

    def test_significant_data_change_via_rms_error(self):
        """Unit test for the significant_data_change_via_rms_error"""
        values_list = self.values_list  # Imports values to be used
        # values_list[0] is the GPS data set
        # values_list[0][x] is a single set of data as [descriptor, units, [data]]
        # values_list[0][x][2] is a single set of data
        data_set_altitude = values_list[0][1][0][8]
        data_set_groundspeed = values_list[0][1][0][9]
        data_sets = [data_set_altitude, data_set_groundspeed]

        constant_value_points_list = []
        mean_list = []
        data_point_list = []
        rms_error_list = []

        sensitivities = [0.05, 0.1, 0.2, 0.5, 0.7]
        for sensitivity in sensitivities:
            for data_set in data_sets:
                constant_value_points, mean, data_point, rms_error = \
                    take_off_detection.significant_data_change_via_rms_error(data_set[2], sensitivity)
                constant_value_points_list.append(constant_value_points)
                mean_list.append(mean)
                data_point_list.append(data_point)
                rms_error_list.append(rms_error)

        lists = [mean_list, data_point_list, rms_error_list]
        expected_data = [[8.388690476190476, 0.0034166666805555555, 8.396235294117647, 0.0034166666805555555,
                          8.410813953488372, 0.01558904110958904, 8.551666666666668, 0.08913333334666666,
                          8.678152173913045, 0.15988157896052632],
                         [84, 72, 85, 72, 86, 73, 90, 75, 92, 76],
                         [0.06874143389818238, 0.10257614404379453, 0.1336248679103539, 0.10257614404379453,
                          0.20551288210907936, 0.22779510268712833, 0.5493127740558753, 0.6086535257834387,
                          0.7469731708072621, 0.8356314669674233]]

        # Checks that all the items generated are correct. The constant_value_points_list is omitted for brevity
        for list_item in range(len(lists)):
            for item in range(len(lists[list_item])):
                self.assertEqual(lists[list_item][item], expected_data[list_item][item])


if __name__ == '__main__':
    unittest.main()



"""
This file is created in order to aid testing.
Author: Adrian Weishaeupl
Created 19/02/2020
"""


def check_str_in_content(string, content):
    """Checks that a certain string is present in the content
    provided. This is case sensitive. This is used in the test functions."""
    if string in content:
        # Checks that the string is present in the content
        occurrence = 0
        # Counts the number of occurrences
        for section in range((len(content) - len(string) + 1)):
            if content[section:section + len(string)] == string:
                occurrence = occurrence + 1
            else:
                continue
    else:
        occurrence = 0
    # Returns results
    return occurrence
